'use strict';

const{mapKeys,difference,intersection}=require("lodash");
const{STATUS_CODES}=require("statuses");

const{TestOpenApiError}=require("../../../errors");
const{getWordsList,merge}=require("../../../utils");
const{
utils:{parseStatus,serializeStatus}}=
require("../../validate");




const addSpecToValidate=function({
validate={},
pluginNames,
operation:{responses,operationId}})
{

if(!pluginNames.includes("validate")){
return;
}

const status=getSpecStatus({validate,responses,operationId});

const responsesA=mapKeys(responses,(value,responseStatus)=>
handleDefaultResponse({responseStatus,responses}));


const validateA=merge({status},validate,responsesA);
return validateA;
};




const getSpecStatus=function({
validate:{status=DEFAULT_STATUS},
responses,
operationId})
{

if(responses.default!==undefined){
return status;
}


const statuses=parseStatus({status,property:"task.validate.status"});

const responseStatuses=Object.keys(responses);

const statusesA=intersection(statuses,responseStatuses);

validateEmptyStatus({
statuses:statusesA,
responseStatuses,
status,
operationId});



const statusA=serializeStatus({statuses:statusesA});
return statusA;
};



const DEFAULT_STATUS="2xx";


const validateEmptyStatus=function({
statuses,
responseStatuses,
status,
operationId})
{
if(statuses.length!==0){
return;
}

const expected=responseStatuses.map(Number);
const responseStatusesStr=getWordsList(responseStatuses);
throw new TestOpenApiError(
`Specification for '${operationId}' describes the following status codes: ${responseStatusesStr} but 'task.validate.status' only allows the following status codes: ${status}`,
{value:status,expected,property:"task.validate.status"});

};



const handleDefaultResponse=function({responseStatus,responses}){
if(responseStatus!=="default"){
return responseStatus;
}

const responseStatuses=Object.keys(responses);
const validStatuses=Object.keys(STATUS_CODES);
const statuses=difference(validStatuses,responseStatuses);

const key=serializeStatus({statuses});
return key;
};

module.exports={
addSpecToValidate};
//# sourceMappingURL=validate.js.map