'use strict';

const{mapValues}=require("lodash");

const{isObject,getPath}=require("../../../utils");
const{checkIsSchema}=require("../../../validation");


const handleJsonSchemas=function({validate}){
return mapValues(validate,handleJsonSchema);
};

const handleJsonSchema=function(value,prop){
if(!hasJsonSchema({prop})){
return value;
}

if(!isObject(value)){
return applyShortcut({value});
}

validateJsonSchema({value,prop});

return value;
};

const hasJsonSchema=function({prop}){
return prop.startsWith("headers.")||prop==="body";
};


const applyShortcut=function({value}){
const type=guessType(value);
return{...type,enum:[value]};
};




const guessType=function(value){
if(value===undefined){
return;
}

const[type]=TYPES.find(([,condition])=>condition(value));
return{type};
};

const TYPES=Object.entries({
null:value=>value===null,
integer:Number.isInteger,
number:value=>typeof value==="number"&&!Number.isInteger(value),
string:value=>typeof value==="string",
boolean:value=>typeof value==="boolean",
array:Array.isArray,

object:()=>true});



const validateJsonSchema=function({value,prop}){
const valueProp=getPath(["task","validate",prop]);
checkIsSchema({value,valueProp});
};

module.exports={
handleJsonSchemas};
//# sourceMappingURL=json_schema.js.map