'use strict';

const{difference,uniq}=require("lodash");

const{TestOpenApiError}=require("../../../../errors");
const{sortArray}=require("../../../../utils");

const{parseRanges,replaceByRanges}=require("./range");
const{VALID_STATUSES}=require("./valid");
const{normalizeStatuses}=require("./normalize");


const parseStatus=function({status,property}){


const statusA=String(status);


const statuses=statusA.split(/[\t-\r \xA0\u1680\u2000-\u200A\u2028\u2029\u202F\u205F\u3000\uFEFF]+/u);

const statusesA=uniq(statuses);

const statusesB=parseRanges({statuses:statusesA});

checkValidStatuses({statuses:statusesB,property});

return statusesB;
};

const checkValidStatuses=function({statuses,property}){
const invalidStatuses=difference(statuses,VALID_STATUSES);

if(invalidStatuses.length===0){
return;
}

const{value,statusesStr}=normalizeStatuses(invalidStatuses);
const expected=VALID_STATUSES.map(Number);
throw new TestOpenApiError(
`The task definition is invalid: those are not valid HTTP status codes: ${statusesStr}`,
{value,expected,property});

};


const serializeStatus=function({statuses}){
const statusesA=replaceByRanges({statuses});
const statusesB=sortArray(statusesA);
const statusKey=statusesB.join(" ");
return statusKey;
};


const STATUS_REGEXP=/^[1-5][0-9x]{2}/iu;

module.exports={
parseStatus,
serializeStatus,
STATUS_REGEXP};
//# sourceMappingURL=parse.js.map