'use strict';

const{addErrorHandler,TestOpenApiError,BugError}=require("../errors");
const{checkSchema}=require("../validation");


const getModule=function(name,info){

if(typeof name!=="string"){
return name;
}

const moduleObj=eLoadModule({name,info});

validateModule({moduleObj,info});

return moduleObj;
};




const loadModule=function({name,info:{corePath}}){

const moduleObj=require(`${corePath}${name}`);
return{...moduleObj,name};
};

const loadModuleHandler=function(
{code,message},
{name,info,info:{title}})
{
checkModuleNotFound({code,name,info});

const props=getProps({info,name});
throw new BugError(
`The ${title} '${name}' could not be loaded: ${message}`,
props);

};





const checkModuleNotFound=function({
code,
name,
info,
info:{title,modulePrefix}})
{
if(code!=="MODULE_NOT_FOUND"){
return;
}

const props=getProps({info,name,addModule:false});
throw new TestOpenApiError(
`The ${title} '${name}' is used in the configuration but is not installed. Please run 'npm install ${modulePrefix}${name}.`,
props);

};

const eLoadModule=addErrorHandler(loadModule,loadModuleHandler);


const validateModule=function({
moduleObj,
moduleObj:{name},
info,
info:{title,schema}})
{
const schemaA=addNameSchema({schema});
const props=getProps({info,name});
checkSchema({
schema:schemaA,
value:moduleObj,
valueProp:title,
message:`the ${title} '${name}' is invalid`,
props,
bug:true});

};




const addNameSchema=function({schema,schema:{properties}}){
return{...schema,properties:{...properties,name:NAME_SCHEMA}};
};

const NAME_SCHEMA={
type:"string",
pattern:"^[a-zA-Z_$][\\w_$-]*$"};



const getProps=function({
info:{props:getErrorProps,title},
name,
addModule=true})
{
const props=getModuleProp({title,name,addModule});

if(getErrorProps===undefined){
return props;
}

const propsA=getErrorProps({name});
return{...props,...propsA};
};

const getModuleProp=function({title,name,addModule}){
if(!addModule){
return;
}

return{module:`${title}-${name}`};
};

module.exports={
getModule};
//# sourceMappingURL=main.js.map