'use strict';

const{readFile}=require("fs");
const{promisify}=require("util");

const fastGlob=require("fast-glob");
const{load:loadYaml,JSON_SCHEMA}=require("js-yaml");
const{sortBy}=require("lodash");

const{addErrorHandler,TestOpenApiError}=require("../errors");

const{validateFileTasks,validateInlineTasks}=require("./validate");
const{addScopes,addKey,validateScopes}=require("./scope");








const loadTasks=async function({tasks}){
const fileTasks=await loadFileTasks({tasks});

const inlineTasks=loadInlineTasks({tasks});

const tasksA=[...fileTasks,...inlineTasks];

const tasksB=tasksA.map(addKey);



const tasksC=sortBy(tasksB,"scope");
return tasksC;
};


const loadFileTasks=async function({tasks}){
const tasksA=tasks.filter(task=>typeof task==="string");


const paths=await fastGlob(tasksA);

validateScopes({paths});

const tasksB=await Promise.all(paths.map(path=>loadTaskFile({path})));
const tasksC=tasksB.flat();
return tasksC;
};


const loadTaskFile=async function({path}){
const content=await eReadFile(path);
const tasks=eParseTaskFile({path,content});

validateFileTasks({tasks,path});

const tasksA=addScopes({tasks,path});
return tasksA;
};

const readFileHandler=function({message},path){
throw new TestOpenApiError(`Could not load task file '${path}': ${message}`);
};

const eReadFile=addErrorHandler(promisify(readFile),readFileHandler);


const parseTaskFile=function({path,content}){
return loadYaml(content,{...YAML_OPTS,filename:path});
};

const YAML_OPTS={
schema:JSON_SCHEMA,
json:true,
onWarning(error){
throw error;
}};


const parseTaskFileHandler=function({message},{path}){
throw new TestOpenApiError(
`Task file '${path}' is not valid YAML nor JSON: ${message}`);

};

const eParseTaskFile=addErrorHandler(parseTaskFile,parseTaskFileHandler);


const loadInlineTasks=function({tasks}){
const tasksA=tasks.filter(task=>typeof task!=="string");

validateInlineTasks({tasks:tasksA});

return tasksA;
};

module.exports={
loadTasks};
//# sourceMappingURL=load.js.map