
'use strict';

const{get}=require("lodash");

const{addErrorHandler}=require("../errors");
const{crawl,promiseThen,promiseAllThen}=require("../utils");

const{parseTemplate,parseEscape}=require("./parse");
const{checkRecursion}=require("./recursion");
const{templateHandler}=require("./error");





















const evalTemplate=function(data,vars={},opts={},stack){
const recursive=recursiveEval.bind(null,vars,opts);
const optsA={...opts,vars,stack,recursive};

return crawl(data,evalNode.bind(null,optsA));
};



const recursiveEval=function(vars,opts,stack,data){
return evalTemplate(data,vars,opts,stack);
};


const evalNode=function(opts,data,path){
const template=parseTemplate(data);


if(template===undefined){
return data;
}

const{type}=template;

if(type==="concat"){
return evalConcat({template,opts,path});
}

return eEvalSingle({template,opts,data,path});
};



const evalConcat=function({template:{tokens},opts,path}){
const maybePromises=tokens.map((token)=>
evalConcatToken({token,opts,path}));



return promiseAllThen(maybePromises,concatTokens);
};

const evalConcatToken=function({token,token:{type,name},opts,path}){

if(type==="raw"){
return name;
}

return eEvalSingle({template:token,opts,data:name,path});
};





const concatTokens=function(tokens){
return tokens.join("");
};

const evalSingle=function({template,opts}){
const unescapedData=parseEscape({template});


if(unescapedData!==undefined){
return unescapedData;
}


const optsA=checkRecursion({template,opts});

const{data,propPath}=getTopLevelProp({template,opts:optsA});




if(data===undefined){
return;
}


return promiseThen(data,(dataA)=>
getNestedProp({data:dataA,template,opts:optsA,propPath}));

};

const eEvalSingle=addErrorHandler(evalSingle,templateHandler);


const getTopLevelProp=function({
template,
template:{name},
opts:{vars}})
{
const{topName,propPath}=parseName({name});

const data=vars[topName];

const dataA=evalFunction({data,template,propPath});

return{data:dataA,propPath};
};




const parseName=function({name}){
const index=name.search(BRACKETS_REGEXP);

if(index===-1){
return{topName:name};
}

const topName=name.slice(0,index);

const delimIndex=getDelimIndex({name,index});
const propPath=name.slice(delimIndex);

return{topName,propPath};
};


const BRACKETS_REGEXP=/[\.\[]/u;


const getDelimIndex=function({name,index}){
if(name[index]==="["){
return index;
}

return index+1;
};




const evalFunction=function({data,template,propPath}){
if(!shouldFireFunction({data,template,propPath})){
return data;
}

return data();
};

const shouldFireFunction=function({data,template:{type},propPath}){
return(
typeof data==="function"&&


Object.keys(data).length===0&&



!(type==="function"&&propPath===undefined));

};


const getNestedProp=function({
data,
template,
opts:{recursive},
propPath})
{









const dataA=recursive(data);

return promiseThen(dataA,(dataB)=>
evalNestedProp({data:dataB,template,propPath}));

};

const evalNestedProp=function({data,template:{type,arg},propPath}){
const dataA=getProp({data,propPath});


if(type!=="function"){
return dataA;
}




const args=Array.isArray(arg)?arg:[arg];



return dataA(...args);
};

const getProp=function({data,propPath}){
if(propPath===undefined){
return data;
}

return get(data,propPath);
};

module.exports={
evalTemplate};
//# sourceMappingURL=eval.js.map